import {Skill, Wish, WishData} from "./datatypes.ts";
import {shuffle} from "./utils.ts";
import {
  bat0, bat1, bat2,
  bat3,
  charm0,
  charm1,
  charm2,
  charm3,
  lore0, lore1, lore2,
  party0,
  party1, party2, party3, stare0, stare1, stare2, stare3,
  stealth0,
  stealth1,
  stealth2,
  stealth3
} from "./skills.ts";

class WishesTable {
  #wishes: WishData[]

  constructor() {
    this.#wishes = [];
  }

  add(data: WishData): Wish {
    let id = this.#wishes.length;
    this.#wishes.push(data);
    return {id};
  }

  get(wish: Wish): WishData {
    return this.#wishes[wish.id];
  }

  getAllPossibleWishes(): Wish[] {
    let wishes: Wish[] = [];
    for (let i = 0; i < this.#wishes.length; i++) {
      wishes.push({id: i});
    }
    return wishes;
  }
}

let table = new WishesTable();
export function getWishes(): WishesTable {
  return table;
}

export const celebritySocialite = table.add({
  profile: {name: "Celebrity Socialite"},
  bannedSkills: () => [lore0],
  discouragedSkills: () => [stealth0, stealth1, stealth2, stealth3],
  encouragedSkills: () => [party0, party1, party2, party3],
  requiredSkills: () => [charm0, charm1, charm2],
});

export const nightswornAlchemist = table.add({
  profile: {name: "Nightsworn Alchemist"},
  bannedSkills: () => [party0],
  discouragedSkills: () => [charm0, charm1, charm2, charm3],
  encouragedSkills: () => [stare0, stare1, stare2, stare3],
  requiredSkills: () => [lore0, lore1, lore2]
});

export const batFreak = table.add({
  profile: {name: "Bat Freak"},
  bannedSkills: () => [charm0, stare0, party0, lore0],
  discouragedSkills: () => [],
  encouragedSkills: () => [stealth0, stealth1, stealth2, stealth3],
  requiredSkills: () => [bat0, bat1, bat2, bat3]
});

export function generateWishes(): Wish[] {
  let possibleWishes = table.getAllPossibleWishes();
  shuffle(possibleWishes);

  let selectedWishes: Wish[] = [];
  for (let i = 0; i < possibleWishes.length; i++) {
    selectedWishes.push(possibleWishes[i]);
  }
  return selectedWishes;
}

export function getCostMultiplier(wish: Wish | null, skill: Skill): number {
  if (wish == null) { return 1.0; }

  let wishData = getWishes().get(wish);

  for (let subj of wishData.requiredSkills()) {
    if (subj.id == skill.id) { return 0.75; }
  }
  for (let subj of wishData.encouragedSkills()) {
    if (subj.id == skill.id) { return 0.875; }
  }

  for (let subj of wishData.discouragedSkills()) {
    if (subj.id == skill.id) { return 1.25; }
  }

  for (let subj of wishData.bannedSkills()) {
    if (subj.id == skill.id) { return 9999.0; }
  }

  return 1.0;
}